

/***********************************
 * TODO:
 * add logic to print human readable tick sizes, i.e.
 * 10 kb instead of 10,000 bp
 */


Vue.component( "axis", {
  template:
    `
    <svg :width="width" :height="height" :viewBox="xOffset +' '+ vOffset +' '+ width + xOffset +' '+ vOffset + height">
        <line x1="0" :y1="2*vOffset" :x2="width" :y2="2*vOffset" style="stroke:rgb(0,0,0);stroke-width:1" />

        <line x1="0" :y1="2*vOffset" x2="0" :y2="vOffset" />
        <line :x1="width" :y1="2*vOffset" :x2="width" :y2="vOffset" />

        <line v-for="tick in ticks" :x1="tick.panelPos" :y1="2*vOffset" :x2="tick.panelPos" :y2="vOffset" />
        <text v-for="tick in ticks" :style="'font: '+vOffset+'px roboto;'"class="small" :x="tick.panelPos" y="0" fill="black">{{ tick.genomePos }}</text>
    </svg>
    `,
  props: {
    start: Number,
    stop: Number,
    genomeSize: Number,
    width: Number,
    height: Number,
    xOffset: Number,
    vOffset: Number
  },
  methods: {
      genomeToPixel( pos ) {
          return pos/Math.abs(this.stop-this.start) * this.width
      }
  },
  mounted(){
    const genomeSectionSize = (this.start > this.stop) ? (this.genomeSize-this.start + this.end) : Math.abs(this.stop-this.start) // genome section in bp
    const tickSize = Math.pow( 10, Math.round( Math.log10( genomeSectionSize / this.noTicks ) ) ) // axis tick size in log 10 bp
    const firstTickPos = Math.pow( 10, Math.ceil( Math.log10( this.start ) ) )

    //let ticks = []
    let genomePos = firstTickPos
    let panelPos  = this.genomeToPixel( genomePos )
    while( panelPos <= this.width ) {
      if( this.start > this.stop  &&  genomePos > this.genomeSize ) {
        genomePos = tickSize
      }
      let tick = {
        genomePos: genomePos,
        panelPos: panelPos
      }
      this.ticks.push( tick )
      genomePos += tickSize
      panelPos  = this.genomeToPixel( genomePos )
    }
  },
  data: function(){
    return {
        noTicks: 10,
        ticks: []
    }
  }
} )

